<?php

namespace HighwayPro\App\Data\Model\DestinationTargets;

use HighWayPro\App\HighWayPro\DestinationComponents\DestinationTargetComponent;
use HighWayPro\App\HighWayPro\HTTP\Response;
use HighWayPro\Original\Data\Drivers\WordPressDatabaseDriver;
use HighwayPro\App\Data\Model\DestinationTargets\DestinationTargetGateway;
use HighwayPro\App\Data\Model\Destinations\DestinationComponentDomain;
use HighwayPro\App\Data\Model\UrlViews\UrlViewGateway;
use HighwayPro\App\HighwayPro\HTTP\Responses\Redirection;
use Highwaypro\App\Data\Model\Preferences\UrlPreferences;

Class DestinationTarget extends DestinationComponentDomain
{
    public function getTargetComponent()
    {
        return $this->getComponent(DestinationTargetComponent::type);
    }

    public function getEvent()
    {
        return $this->getTargetComponent()->getEvent();  
    }

    public function send()
    {
        (object) $targetComponent = $this->getTargetComponent();
        (object) $response = $this->expect($targetComponent->response())->toBe(Response::class);

        $this->setResponsePreferences($response);

        $response->send();

        $this->shutDown($response);
    }

    protected function setResponsePreferences(Response $response)
    {
        if ($response instanceof Redirection) {
            (object) $url = $this->getDestination()->getUrl();

            $response->withStatusCode($url->getRedirectionStatusCode());
            $response->setRedirectUsingHTML($url->needsToPerformRedirectionUsingHTML());
            $response->setReferrerType($url->getReferrerType());
            $response->setReferrerPolicy($url->getReferrerPolicy());
            $response->setTrackingScripts($url->getTrackingScripts());
            
            if ($url->shouldBeMarkedAsNoIndex()) {
                $response->withHeaders([
                    'X-Robots-Tag' => 'noindex, nofollow'
                ]);
            }
            // will be available in the future...
            /*$response->withHeaders([
                'Referrer-Policy' => $this->getDestination()->getUrl()->getRedirectionRefererType()
            ]);*/
        }   
    }
    

    protected function shutDown(Response $response)
    {
        register_shutdown_function(function() use ($response) {
            $this->onShutDown($response);
        });

        if ($response->needsToExit()) {
            exit;
        }   
    }
 
    protected function onShutDown(Response $response)
    {
        $this->saveView($response);

        $this->getTargetComponent()->onShutDown($this);
    }

    protected function saveView(Response $response)
    {
        if ($response->shouldSaveView()) {
            (new UrlViewGateway(new WordPressDatabaseDriver))->saveView(
                $this->destination->createView()
            );
        }    
    }


    /**
     * This method simply uses Gateway for saving the model in the database. This is the first move towards
     * implementing a save() method in the Domain base, that'll still rely on Gateway for database access.
     */
    public function save()
    {
        (object) $destinationTargetGateway = new DestinationTargetGateway(new WordPressDatabaseDriver);

        $result = $destinationTargetGateway->deleteAllWithDestinationIdAndInsertNew(
            new DestinationTarget(
                array_merge(
                    $this->getAvailableValues(),
                    [
                        'parameters' => $this->parameters
                    ]
                )
            )
        );   
    }
    
    
}