<?php

namespace Highwaypro\App\Data\Model\Preferences;

use HighWayPro\Original\Collections\Collection;
use HighwayPro\App\HighwayPro\HTTP\Responses\Redirection;
use HighwayPro\Original\Collections\JSONMapper;
use HighwayPro\Original\Collections\Mapper\Mappable;
use HighwayPro\Original\Collections\Mapper\Types;
use Highwaypro\App\Data\Model\Preferences\Preferences;

Class UrlPreferences extends Preferences
{
    const CHARACTER_ALPHABETIC   = 'alphabetical';
    const CHARACTER_NUMERIC      = 'numeric';
    const CHARACTER_ALPHANUMERIC = 'alphanumeric';

    const CASE_ANY       = 'any';
    const CASE_UPPERCASE = 'uppercase';
    const CASE_LOWERCASE = 'lowercase';

    const REFERER_TYPE_ALWAYS = 'always';
    const REFERER_TYPE_ORIGIN   = 'domain';
    const REFERER_TYPE_HIDE  = 'hide';

    const NO_INDEX_MODE_NONE = 'none';
    const NO_INDEX_MODE_ALL = 'all';
    const NO_INDEX_MODE_ONLY_IN_TYPES = 'only_in_types';

    const PARAMETER_FORWARDING_MODE_ALL = 'all';
    const PARAMETER_FORWARDING_MODE_INCLUDE_ONLY = 'include';
    const PARAMETER_FORWARDING_MODE_EXCLUDE = 'exclude';

    static public function fields()
    {
        return new Collection([
        'generateUrlsOnNewPosts_isEnabled'  => Types::BOOLEAN()->withDefault(true),
        'postUrl_UrlType'                   => Types::INTEGER(), // the base path of post urls
        'autoGeneratedPathType'             => Types::STRING()->withDefault(self::CHARACTER_NUMERIC) 
                                               ->allowed([
                                                    self::CHARACTER_ALPHABETIC, 
                                                    self::CHARACTER_NUMERIC, 
                                                    self::CHARACTER_ALPHANUMERIC
                                                ]),
        'autoGeneratedPathAlphanumericCase' => Types::STRING()->withDefault(self::CASE_ANY) 
                                               ->allowed([
                                                    self::CASE_ANY, 
                                                    self::CASE_UPPERCASE, 
                                                    self::CASE_LOWERCASE
                                                ]),
        'autoGeneratedPathLength'           => Types::INTEGER()->withDefault(6)->minimum(2), 
        'http_redirection_status_code'      => Types::INTEGER()->withDefault(Redirection::DEFAULT_STATUS_CODE)
                                                               ->minimum(Redirection::DEFAULT_STATUS_CODE_MINIMUM)
                                                               ->allowed([
                                                                    'Multiple Choice' => 300,
                                                                    'Moved Permanently' => 301,
                                                                    'Found' => 302,
                                                                    'See Other' => 303,
                                                                    'Not Modified' => 304,
                                                                    'Temporary Redirect' => 307,
                                                                    'Permanent Redirect' => 308
                                                                ]),
        'http_referrer_type'                => Types::STRING()->withDefault(self::REFERER_TYPE_ALWAYS) 
                                                              ->allowed([
                                                                   'Always send it' => self::REFERER_TYPE_ALWAYS, 
                                                                   'Only the domain' => self::REFERER_TYPE_ORIGIN, 
                                                                   'Hide it' => self::REFERER_TYPE_HIDE,
                                                               ]),
        'http_no_index_mode'                => Types::STRING()->withDefault(self::NO_INDEX_MODE_NONE)
                                                                  ->allowed([
                                                                      'Index All' => self::NO_INDEX_MODE_NONE,
                                                                      'Set Noindex to all URLs' => self::NO_INDEX_MODE_ALL,
                                                                      'Set Noindex to URLs with these types...' => self::NO_INDEX_MODE_ONLY_IN_TYPES
                                                                  ]),
        'http_no_index_type_ids'            => Types::COLLECTION(),
        'http_parameter_forwarding_is_enabled' => Types::BOOLEAN()->withDefault(false),
        'http_parameter_forwarding_mode' => Types::STRING()->withDefault(self::PARAMETER_FORWARDING_MODE_ALL)
                                                            ->allowed([
                                                                'All parameters' => self::PARAMETER_FORWARDING_MODE_ALL,
                                                                'Only these parameters...' => self::PARAMETER_FORWARDING_MODE_INCLUDE_ONLY,
                                                                'All except these parameters...' => self::PARAMETER_FORWARDING_MODE_EXCLUDE
                                                            ]),
        'http_parameter_forwarding_parameter_names' => Types::COLLECTION()->separatedBy("\n"),
        'hp_global_tracking_scripts' => Types::STRING()->escape(Types::returnValueCallable())
        ]);   
    }

    protected function getMap()
    {
        return static::fields()->asArray();
    }

    public function __construct($data)
    {
        $this->preferences = $this->map($data);
    }
}