<?php

namespace HighWayPro\App\Data\Model\UrlViews\UrlViewStatistics;

use HighWayPro\App\HighwayPro\Time\PeriodOfTime;
use HighWayPro\Original\Collections\Collection;
use HighwayPro\App\Data\Model\UrlViews\UrlViewGateway;
use HighwayPro\App\HighwayPro\System\Date;

Class AllUrlViewStatistics extends UrlViewStatisticsData
{
    protected function getDataToExport()
    {
        (object) $today = $this->urlViewStatistics->getCurrentDate();

        return new Collection([
            'viewsType' => $this->getType(),
            'urlId' => $this->urlViewStatistics->getOptionalUrlId(),
            'count' => [
                'allTime' => $this->urlViewStatistics->getTotalAllTime(),
                'today' => $this->urlViewStatistics->getTotalInDate(
                    $period = PeriodOfTime::createFromPreset('today', $today),
                    $periodVsPeriodDifference = PeriodOfTime::createFromPreset('lastDay', $today)
                ),
                'lastDay' => $this->urlViewStatistics->getTotalInDate(
                    $period = PeriodOfTime::createFromPreset('lastDay', $today),
                    $periodVsPeriodDifference = PeriodOfTime::createFromPreset('twoDaysAgo', $today)
                ),
                'thisMonth' => $this->urlViewStatistics->getTotalInDate(
                    $period = PeriodOfTime::createFromPreset('thisMonth', $today),
                    $periodVsPeriodDifference = PeriodOfTime::createFromPreset('samePeriodOfLastMonth', $today)
                ),
                'thisYear' => $this->urlViewStatistics->getTotalInDate(
                    $period = PeriodOfTime::createFromPreset('thisYear', $today),
                    $periodVsPeriodDifference = PeriodOfTime::createFromPreset('samePeriodOfLastYear', $today)
                ),
            ],
            'inPeriod' => [
                //'dailyCount'
                'periodCount' => [
                    'length' => $this->urlViewStatistics->getPeriodOfTime()->getLength(), // 28/29/30/31 (days) or 24 (hours) or 6 (weeks) or 3 (months)
                    'unit'   => $this->urlViewStatistics->getPeriodOfTime()->getUnit(), //days or hours or weeks or moths
                    'count' => $this->urlViewStatistics->getCount(), // getCountFromPastDays(30),
                    'total' => $this->urlViewStatistics->getTotal(), // totral in the current period eg: 96 clicks
                    'date' => [
                        'from' => $this->urlViewStatistics->getPeriodOfTime()->getStartDateFormattedWithTimeStamp(),
                        'to' => $this->urlViewStatistics->getPeriodOfTime()->getEndDateFormattedWithTimeStamp()
                    ]
                ],
                'countByField' => [
                    'countries' => $this->urlViewStatistics->getTotalBy('location_country'),
                    'devices' => $this->urlViewStatistics->getTotalBy('device_type'),
                    'origin' => $this->urlViewStatistics->getTotalBy('device_referer'),
                    'operativeSystem' => $this->urlViewStatistics->getTotalBy('device_os'),
                    'language' => $this->urlViewStatistics->getTotalBy('location_language'),
                ]
            ]
        ]);
    }
}