<?php

namespace HighWayPro\App\Data\Model\UrlViews\UrlViewStatistics;

use HighWayPro\App\Data\Model\UrlViews\UrlViewStatistics\AllUrlViewStatistics;
use HighWayPro\App\HighwayPro\Time\PeriodOfTime;
use HighWayPro\Original\Collections\Collection;
use HighWayPro\Original\Data\Drivers\WordPressDatabaseDriver;
use HighWayPro\Original\Utilities\TypeChecker;
use HighwayPro\App\Data\Model\UrlViews\UrlViewGateway;
use HighwayPro\App\Data\Model\Urls\Url;
use HighwayPro\App\HighwayPro\System\Date;

Class UrlViewStatistics implements UrlViewStatisticsInterface
{
    use TypeChecker;

    protected $urlViewGateway;
    protected $urlId;
    protected $currentDate;

    public function __construct(Array $data = null)
    {
        $data = new Collection($data?$data:[]);

        $this->urlId = $data->get('urlId');
        $this->periodOfTime = $this->expect($data->get('periodOfTime'))->toBe(PeriodOfTime::class);
        $this->currentDate = $this->expect($data->get('currentDate'))->toBe(Date::class);
        $this->urlViewGateway = new UrlViewGateway(new WordPressDatabaseDriver);
    }    

    public function getPeriodOfTime()
    {
        return $this->periodOfTime;   
    }

    public function getCurrentDate()
    {
        return $this->currentDate;   
    }

    public function getStatistics()
    {
        return new AllUrlViewStatistics($this);
    }

    public function getTotalAllTime()
    {
        return $this->getTotal($inCurrentPeriod = false);
    }
    
    /**
     * Gets the total count (number) IN THE GIVEN PERIOD
     * @return int
     */
    public function getTotal($inCurrentPeriod = true)
    {
        return $this->urlViewGateway->getTotalNumberOfRows(
            $this->getOptionalUrlId(), 
            $inCurrentPeriod? $this->getPeriodOfTime() : null
        );
    }

    public function getTotalInDate(PeriodOfTime $period, PeriodOfTime $periodToCompare = null)
    {
        return $this->urlViewGateway->getTotalNumberOfRowsInPeriodWithDifference(
            $period, 
            $periodToCompare,
            $this->getOptionalUrlId()
        );
    }

    public function getTotalBy($field)
    {
        return $this->urlViewGateway->getTotalCountByField($field, $this->periodOfTime, $this->getOptionalUrlId());
    }

    public function getCount()
    {
        return $this->urlViewGateway->getCountInPeriod($this->periodOfTime, $this->getOptionalUrlId());
    }

    public function isForSingleUrl()
    {
        return is_numeric($this->urlId);   
    }
    
    public function getOptionalUrlId()
    {
        if ($this->isForSingleUrl()) {
            return $this->urlId;
        }
    }
}