<?php

namespace HighWayPro\App\HighWayPro\DestinationComponents\BuiltIn\Conditions;

use HighWayPro\App\HighWayPro\HTTP\Request;
use HighWayPro\Original\Collections\Mapper\Types;

Class LanguageCondition extends DestinationConditionComponent
{
    const name = 'highwaypro.LanguageCondition';

    public static function title()
    {
        return __('Language', Env::textDomain());
    }

    public static function shortDescription()
    {
        return __('Target users by their browser language', Env::textDomain());
    }

    public static function description()
    {
        return __("Target users by the language of their web browser.", Env::textDomain());
    }

    protected function getAllowedValues()
    {
        return new Collection([
            'countries' => Countries::all()
        ]);
    }

    protected function parametersMap()
    {
        return [
            'languages' => Types::Collection(),
        ];
    }

    public function hasPassed()
    {
        (object) $request = Request::instance();


        (object) $ipDatabaseReader = new LocationDetector($request->getClientIP());

        return $this->parameters->countries->contain(
            $ipDatabaseReader->getCountryCode()
        );
    }

    public function validateParameters()
    {
        if (!Countries::all()->getKeys()->containAll($this->parameters->countries)) {
            return LocationConditionErrors::get('invalid_countries_selected');
        }

        return true;
    }
}