<?php

namespace HighWayPro\App\HighWayPro\DestinationComponents;

use Exception;
use HighWayPro\App\Data\Schema\DestinationTargetTable;
use HighWayPro\App\HighWayPro\DestinationComponents\DestinationConditionComponent;
use HighWayPro\App\HighWayPro\DestinationComponents\DestinationTargetComponent;
use HighWayPro\App\HighWayPro\HTTP\Response;
use HighWayPro\App\HighWayPro\Validators\Url\Errors\DestinationTargetErrors;
use HighWayPro\Original\Cache\MemoryCache;
use HighWayPro\Original\Characters\StringManager;
use HighWayPro\Original\Collections\Collection;
use HighwayPro\App\Data\Model\Destinations\Destination;
use HighwayPro\App\Data\Model\Urls\Url;
use HighwayPro\Original\Collections\JSONMapper;
use HighwayPro\Original\Collections\Mapper\Types;
use StdClass;

Abstract Class DestinationComponent
{
    const event = 'plugins_loaded';

    protected $parameters;
    protected $allowed;
    protected $cache;
    
    abstract protected function parametersMap();
    abstract public function validateParameters();

    protected function getAllowedValues() { return new Collection([]); }

    public static function getMetaData()
    {
        return new Collection([
            'title' => static::title(),
            'shortDescription' => static::shortDescription(),
            'description' => static::description(),
            'parametersMap' => (new static(''))->parameters,
            'allowedValues' => (new static(''))->allowed
        ]);   
    }
    
    protected function defaultMap()
    {
        return [
            'queryString' => Types::STRING()->escape(Types::returnValueCallable())
        ];   
    }

    public function __construct($parameters, Destination $destination = null)
    {
        (object) $JSONMapper = new JSONMapper(
            array_merge($this->parametersMap(), $this->defaultMap())
        );

        $this->allowed = $this->getAllowedValues();
        $this->parameters = $JSONMapper->map($parameters);
        $this->destination = $destination? $destination : null;
        $this->cache = new MemoryCache;
    }

    public function getEvent()
    {
        return static::event;   
    }

    public function getParameterValidationResult()
    {
        (boolean) $parametersExceedAllowedlength = 
            $this->parameters->asCollection()->asJson()->length() >= DestinationTargetTable::PARAMETERS_LENGTH;
            
        if ($parametersExceedAllowedlength) {
            return DestinationTargetErrors::get('allowed_parameter_size_exceeded');
        }   

        return $this->validateParameters();
    }

    public function getParameters()
    {
        return clone $this->parameters;   
    }

    public function getAllowed()
    {
        return clone $this->allowed;   
    }

    public function getUrlWithScheme($url)
    {
        (object) $urlComponents = new Collection(parse_url((string) $url));
        
        if (!$urlComponents->hasKey('scheme')) {
            return "//{$url}";
        }

        return (string) $url;
    }
}



