<?php

namespace HighWayPro\App\HighWayPro\HTTP\Requests\UrlView;

use HighWayPro\App\HighWayPro\HTTP\Request;
use HighWayPro\App\HighwayPro\Time\PeriodOfTime;
use HighWayPro\App\HighwayPro\Time\PeriodOfTimePresets;
use HighWayPro\Original\Collections\Mapper\Types;
use HighwayPro\App\Data\Model\Urls\Url;
use HighwayPro\App\HighwayPro\System\Date;
use Highwaypro\App\Data\Model\Preferences\AnalyticsPreferences;

Class UrlViewStatisticsReadRequest extends Request
{
    public function map()
    {
        return [
            'url' => Url::fields()->only(['id'])->asArray(),
            'periodOfTime' => [
                'preset' => Types::STRING()->allowed(PeriodOfTimePresets::getPresets()),
                                               //->withDefault(AnalyticsPreferences::getDefaultPreset()),
                'unit' => Types::STRING()->allowed(PeriodOfTime::$validUnits)->withDefault('auto'),
                'customRange' => [
                    'from' => Types::STRING(),
                    'to' => Types::STRING()
                ]
            ]
        ];
    }

    public function hasUrlObject()
    {
        return $this->data->mapFieldsFoundInSource->contain('url');   
    }

    public function hasUrlId()
    {
        return $this->hasUrlObject() &&
               $this->data->url->mapFieldsFoundInSource->contain('id');
    }

    public function hasInvalidPeriodOfTimePreset()
    {
        return !PeriodOfTimePresets::getPresets()->contain($this->data->periodOfTime->preset->get());   
    }

    public function rangeIsInvalid()
    {
        // Carbon will throw an expection if dates are invalid
        (object) $from = Date::createFromString((string) $this->data->periodOfTime->customRange->from);
        (object) $to = Date::createFromString((string) $this->data->periodOfTime->customRange->to);   

        return $from->greaterThan($to);       
    }

    public function getPeriodOfTimeUnit()
    {
        // auto defaults to null, which tells periodOfTime to figure the unit by itself
        if ($this->data->periodOfTime->unit->isNot('auto')) {
            return $this->data->periodOfTime->unit->get();
        }   
    }
}