<?php

namespace HighwayPro\App\HighwayPro\HTTP\Responses;

use HighWayPro\App\Components\HTMLRedirectionMarkup;
use HighWayPro\App\HighWayPro\HTTP\Response;

Class Redirection extends Response
{
    CONST DEFAULT_STATUS_CODE         = 302;
    CONST DEFAULT_STATUS_CODE_MINIMUM = 300;
    
    public $statusCode                = 302;
    public $redirectionUrl;
    protected $redirectUsingHTML;
    protected $referrerType;
    protected $referrerPolicy;

    protected $cacheControl = 'max-age=10';

    public function __construct(array $data = [])
    {
        parent::__construct($data);

        $this->withHeaders([
            'Cache-Control' => $this->cacheControl
        ]);
    }
    

    public function to($url)
    {
        $this->redirectionUrl = (string) $url;
        $this->headers['Location'] = $url;

        return $this;
    }

    public function setRedirectUsingHTML($redirectUsingHTML)
    {
        $this->redirectUsingHTML = $redirectUsingHTML;

        if ($this->redirectUsingHTML) {
            $this->withStatusCode(200);
            unset($this->headers['Location']);
        }
    }

    public function setReferrerType($referrerType)
    {
        $this->referrerType = $referrerType;   
    }

    public function setReferrerPolicy($referrerPolicy)
    {
        $this->referrerPolicy = $referrerPolicy;
    }

    public function setTrackingScripts($trackingScripts)
    {
        $this->trackingScripts = $trackingScripts;   
    }

    protected function beforeSend()
    {
        if ($this->redirectUsingHTML) {
            $this->body = $this->getHTMLRedirectionMarkup();
        } else {
            $this->body = $this->getHTTPRedirectionMarkup();
        }
    }

    protected function getHTMLRedirectionMarkup()
    {
        (object) $HTMLRedirection = new HTMLRedirectionMarkup([
            'targetURL' => $this->redirectionUrl,
            'referrerPolicy' => $this->referrerPolicy,
            'referrerType' => $this->referrerType,
            'trackingScripts' => $this->trackingScripts
        ]);

        return $HTMLRedirection->getRenderedMarkup();
    }
    

    protected function getHTTPRedirectionMarkup()
    {
        return <<<BODY
<!DOCTYPE html>
<html>
<head>
    <title>Redirecting...</title>
    <meta http-equiv="refresh" content="0; url={$this->redirectionUrl}" />
</head>
<body>

</body>
</html>
BODY;
    }
    
}