<?php

namespace HighwayPro\App\HighwayPro\System;

use Carbon\Carbon;
use DateTimeZone;
use HighWayPro\Original\Collections\Collection;
use HighWayPro\Original\Environment\Env;
use HighwayPro\App\HighwayPro\System\WpDateTimeZone;

Class Date extends Carbon
{
    const FORMAT = 'Y-m-d H:i:s';
    const FORMAT_NO_TIMESTAMP = 'Y-m-d';
    const FORMAT_YEAR_ONLY = 'Y';
    const FORMAT_MONTH_ONLY = 'm';
    const FORMAT_DAY_ONLY = 'd';
    const FORMAT_HOUR_ONLY = 'H';

    const HOUR_START = 0;
    const HOUR_END = 23;

    public static function createFromString($date)
    {
        $date = strpos($date, ':') === false? "{$date} 00:00:00" : $date;

        return static::createFromFormat(static::FORMAT, $date, WpDateTimeZone::getWpTimezone());
    }

    public static function createFromSettings()
    {
        if (Env::isRemoteTesting()) {
            $customDate = apply_filters('highwaypro_develoment_set_date', null);

            if ($customDate) {
                return $customDate;
            }
        }

        return static::createFromFormat(static::FORMAT, current_time(static::FORMAT));   
    }

    /**
     * Generates a range of day hours (from 0 to 23), taking into account next-day ranges,
     * for example, from 22 to 5 (cross-day), the resutl wold be: [22, 23, 0, 1, 2, 3, 4, 5]
     * 
     * @param  int $start
     * @param  int $end
     * @return Collection
     */
    public static function getDayHoursRange($start, $end)
    {
        if ($start <= $end) {
            return Collection::range($start, $end);
        }

        return Collection::range($start, static::HOUR_END)->concat(Collection::range(static::HOUR_START, $end));
    }
    
    /**
     * Gets the hour of the date from 0 to 23
     * @return int
     */
    public function getHour()
    {
        return (integer) $this->format('G');
    }
    
}