<?php

namespace HighWayPro\App\HighWayPro\Validators\Url\Read;

use Exception;
use HighWayPro\App\HighWayPro\HTTP\Requests\UrlView\UrlViewStatisticsReadRequest;
use HighWayPro\App\HighWayPro\HTTP\Response;
use HighWayPro\App\HighWayPro\HTTP\Validator;
use HighWayPro\App\HighWayPro\Validators\Url\Errors\UrlViewsErrors;
use HighWayPro\App\HighwayPro\Time\PeriodOfTimePresets;
use HighWayPro\Original\Data\Drivers\WordPressDatabaseDriver;
use HighwayPro\App\Data\Model\UrlTypes\UrlTypeGateway;
use HighwayPro\App\Data\Model\Urls\Url;
use HighwayPro\App\Data\Model\Urls\UrlGateway;
use HighwayPro\App\Data\Model\Urls\Validators\UrlValidator;
use HighwayPro\App\HighwayPro\System\Date;

Class UrlViewStatisticsReaderValidator extends Validator
{
    protected function request()
    {
        return new UrlViewStatisticsReadRequest;
    }

    public function validate()
    {
        (object) $urlValidator = new UrlValidator(
            new Url([
                'id' => $this->request->data->url->id
            ]),
            new UrlGateway(new WordPressDatabaseDriver),
            new UrlTypeGateway(new WordPressDatabaseDriver)
        );

        if ($this->request->hasUrlObject() && $urlValidator->idDoesNotExist()) {
            return (new Response)->withStatusCode(404)
                                 ->containing(UrlViewsErrors::get('url_view_statistics_invalid_url_id')->asArray());

        } elseif (!$this->request->data->mapFieldsFoundInSource->contain('periodOfTime')) {
            return (new Response)->withStatusCode(400)
                                 ->containing(UrlViewsErrors::get('url_view_statistics_empty_period_of_time')->asArray());

        } elseif (!$this->request->data->periodOfTime->mapFieldsFoundInSource->contain('preset') 
              || !$this->request->data->periodOfTime->mapFieldsFoundInSource->contain('unit')) {

            (string) $missingField = !$this->request->data->periodOfTime->mapFieldsFoundInSource->contain('preset') ? 'preset' : 'unit';
            return (new Response)->withStatusCode(400)
                                 ->containing(UrlViewsErrors::getCustomMessage([
                                        'type' => 'url_view_statistics_mising_period_of_time_field',
                                        'message' => "Missing required PeriodOfTime field: {$missingField}"
                                     ]));

        } elseif ($this->request->hasInvalidPeriodOfTimePreset()) {
            return (new Response)->withStatusCode(400)
                                 ->containing(UrlViewsErrors::get('url_view_statistics_invalid_preset')->asArray());

        } elseif ($this->request->data->periodOfTime->preset->is(PeriodOfTimePresets::CUSTOM_RANGE)) {
            try {
                if ($this->request->rangeIsInvalid()) {
                    throw new Exception;
                }
                // to less then from etc
                // then uint test it oh yes sir
            } catch (Exception $exception) {
                return (new Response)->withStatusCode(400)
                                     ->containing(UrlViewsErrors::getCustomMessage([
                                        'type' => 'url_view_statistics_invalid_range',
                                        'message' => "Invalid PeriodOfTime custom range date. {$exception->getMessage()}"
                                     ]));
            }
        }

        return true;
    }

}
